//
//  SSCircularSlider.h
//  SSCircularSlider
//
//  Created by Ben Haller on Thu Jan 24 2002.
//  Copyright (c) 2002 Stick Software. All rights reserved.
//
//  This is version 1.3 of SSCircularSlider.
//  Thanks to David Remahl for his contributions.

/*
	Circular Slider (and thus this source file) is open source software, licensed under the terms of the Artistic License.
	That license should be included with the distribution containing this file, but it should also be available at
	http://www.sticksoftware.com/software/artisticLicense.html.  Circular Slider's home page, with the latest version and
	other information, is at http://www.sticksoftware.com/software/CircularSlider.html.
*/

#import <AppKit/AppKit.h>

// Standard sizes:
//
// 80x80, including a 10-pixel margin on all sides for the ball to hang off and for shadow
// 60x60, including a 10-pixel margin on all sides for the ball to hang off and for shadow

typedef enum _SSCircularSliderAppearance {
	SSInactiveTintedAppearance = -1,	// for internal use only; use setEnabled:NO to disable the control
	
    SSSystemTintedAppearance = 0,
    SSBlueTintedAppearance,
    SSGraphiteTintedAppearance,
    SSQuartzAppearance
} SSCircularSliderAppearance;

typedef enum _SSCircularSliderSnapBehavior {
    SSSnapNever = 0,
    SSSnapAlways,
    SSSnapOnShift
} SSCircularSliderSnapBehavior;

typedef enum _SSCircularSliderAngleUnits {
    SSAnglesAreRadians = 0,
    SSAnglesAreDegrees
} SSCircularSliderAngleUnits;

@interface SSCircularSlider : NSView
{
    // Target/action support
    id _target;
    SEL _action;
    BOOL _continuous;
    
    // Enabling and setup
    BOOL _enabled;
    BOOL _allowsCenter, _allowsNonEdge;
    float _maxDistance;
    int _ballRadius;
    SSCircularSliderAppearance _appearance;
    
    // Control value
    float _angle, _distance;
	SSCircularSliderAngleUnits _angleUnits;		// this does not govern _angle, which is always in radians; it governs the units used when the user gets and sets our angle
	
    // Snap support
    SSCircularSliderSnapBehavior _snapBehavior;
    int _snapMarks;

    // Visual snap indication support
    SSCircularSliderSnapBehavior _drawSnapBehavior;
    
    // Tracking
    BOOL _ballHighlighted;
}

+ (NSImage *)baseImageWithCenter:(BOOL)center nonEdge:(BOOL)nonEdge big:(BOOL)big;
+ (NSImage *)knobImageWithTint:(SSCircularSliderAppearance)tint highlighted:(BOOL)highlighted big:(BOOL)big;

// Init
- (id)initWithFrame:(NSRect)frame;
- (id)initWithCoder:(NSCoder *)coder;

// Coding
- (void)encodeWithCoder:(NSCoder *)coder;

// Enable/disable and appearance
- (BOOL)isEnabled;
- (void)setEnabled:(BOOL)flag;

// Basic setup
- (BOOL)allowsCenterPoint;
- (void)setAllowsCenterPoint:(BOOL)flag;
- (BOOL)allowsNonEdgePositions;
- (void)setAllowsNonEdgePositions:(BOOL)flag;
- (float)maximumDistance;
- (void)setMaximumDistance:(float)maxDistance;
- (SSCircularSliderAngleUnits)angleUnits;
- (void)setAngleUnits:(SSCircularSliderAngleUnits)units;

// Getting and setting the control value in angle/distance form
- (void)setAngle:(float)angle andDistance:(float)distance;
- (void)getAngle:(float *)angle andDistance:(float *)distance;

// Getting and setting the control value in xOffset/yOffset form
- (void)setXOffset:(float)xOffset andYOffset:(float)yOffset;
- (void)getXOffset:(float *)xOffset andYOffset:(float *)yOffset;

// Action support
- (id)target;
- (void)setTarget:(id)anObject;
- (SEL)action;
- (void)setAction:(SEL)aSelector;
- (BOOL)isContinuous;
- (void)setContinuous:(BOOL)flag;
- (BOOL)sendAction:(SEL)theAction to:(id)theTarget;
- (BOOL)sendAction;

// Snap support
- (void)setSnapBehavior:(SSCircularSliderSnapBehavior)behavior;
- (SSCircularSliderSnapBehavior)snapBehavior;
- (void)setNumberOfSnapMarks:(int)numSnapMarks;
- (int)numberOfSnapMarks;

- (void)setDrawTicksBehavior:(SSCircularSliderSnapBehavior)behavior;
- (SSCircularSliderSnapBehavior)drawTicksBehavior;

// Tinting
- (SSCircularSliderAppearance)appearance;
- (void)setAppearance:(SSCircularSliderAppearance)appearance;
- (void)controlTintChanged:(NSNotification *)note;		// called by the system

// Constraints
- (BOOL)constrainAngle:(float *)angle andDistance:(float *)distance;	// always in radians
- (BOOL)constrainState;

@end
